<?php
/**
 * Zend Frameworkを利用したRSS/Atom Reader『ZFReader』
 * Feedテーブルクラス
 * 
 * @author ishinao <ishinao@ishinao.net>
 * $Id: Feed.php 73 2006-05-16 06:22:36Z ishinao $
 */

/**
 * Zend_Db_Tableクラスを読み込み
 */
Zend::loadClass('Zend_Db_Table');

/**
 * Feedクラスで生成する例外クラスを定義
 */
class Feed_AlreadyRegisteredException extends Exception {} // URLの重複登録
class Feed_InvalidIdException extends Exception {} // 不正なフィードIDに対する処理

/**
 * feedテーブルに対応するO/Rマッピングクラス
 */
class Feed extends Zend_Db_Table
{
	/**
	 * insertメソッドをoverride
	 */
	public function insert(&$data)
	{

		// 日時情報を自動的にセット
		$data['created'] = date('Y-m-d H:i:s');
		$data['modified'] = date('Y-m-d H:i:s');
		$data['crawled'] = date('Y-m-d H:i:s');

		return parent::insert($data); // 親クラスのinsertを実行

	}
	
	/**
	 * updateメソッドをoverride
	 */
	public function update(&$data, $where)
	{

		// 日時情報を自動的にセット
		$data['modified'] = date('Y-m-d H:i:s');

		return parent::update($data, $where); // 親クラスのupdateを実行
	}
	
	/**
	 * フィードURL登録メソッド
	 * 
	 */
	public function add($url)
	{
		
		if ($this->_isRegistered($url)) {
			throw new Feed_AlreadyRegisteredException('登録済みURLです: ' . $url);
		}

		Zend::loadClass('Zend_Feed'); // Zend_Feedクラスを読み込み
		
		// Zend_Feedを使ってフィードURLからフィード情報を取得
		try {

			// URLが不正な場合は、Zend_Feedから例外が投げられる
		   	$feed = Zend_Feed::import($url);

		} catch (Exception $e) {

			$feeds = Zend_Feed::findFeeds($url); // RSS/Atom Auto Discovery

	    	if (count($feeds) == 0) {
	    		throw $e;
	    	}
	    	
	    	$feed = $feeds[0];
	    	$url = $feed[0]['about'];

			if ($this->_isRegistered($url)) {
				throw new Feed_AlreadyRegisteredException('登録済みURLです: ' . $url);
			}

		}

    	$row = $this->fetchNew(); // 新規Feedテーブル行を作成

    	$row->title = $feed->title(); // タイトルをセット

    	$link = $feed->link(); // フィード情報が提供するWebページのURLを取得
    	if (count($link) > 1) {$link = $link[0]->nodeValue;} // 複数linkノード対策
    	$row->url = $link; // WebページURLをセット

    	$row->feedurl = $url; // フィードURLをセット

    	return $row->save(); // 新規行をDBに保存

	}

	/**
	 * URLが登録済みかどうかを返す
	 * 
	 * @param string $url フィードもしくはWebページのURL
	 * @return boolean 登録済みの場合はtrue
	 */
	protected function _isRegistered($url)
	{
		
    	$db = $this->getAdapter(); // Zend_Db_Adapterオブジェクトを取得（quoteIntoを使うため）
    	
    	// WebページURLとして登録されているか
    	if ($this->fetchAll($db->quoteInto('url = ?', $url))->count() > 0) {
    		return true;
    	}

		// フィードURLとして登録されているか
    	if ($this->fetchAll($db->quoteInto('feedurl = ?', $url))->count() > 0) {
    		return true;
    	}

    	return false;

	}
	
	/**
	 * 有効なフィードIDかどうかを返す
	 * 
	 * @param integer $id フィードID
	 * @return boolean 有効な場合はtrue
	 */
	public function exists($id)
	{

		$feed = $this->find($id); // フィードIDに対応するフィード情報を取得
		return $feed->id == $id; // 正しい情報が取得できたらtrueを返す

	}
	
	/**
	 * フィードの最新アイテム情報を取得する
	 * 
	 * @param integer $id フィードID
	 */
	public function crawlFeed($id)
	{

		$row = $this->find($id); // DBからフィード情報を取得

		if ($row->id != $id) {throw new Feed_InvalidIdException('フィードIDが不正です: ' . $id);}

		$row->crawled = date('Y-m-d H:i:s'); // 最終情報取得時間を更新
		$row->save(); // DBに保存
		
		$feedurl = $row->feedurl; // フィードのURL

		Zend::loadClass('Zend_Feed'); // Zend_Feedクラスを読み込み
		$feed = Zend_Feed::import($feedurl); // フィードを読み込み
		$itemTable = new Item(); // Itemテーブルを作成

		foreach ($feed as $item) {
			$itemTable->addItem($id, $item); // 読み込んだフィードのアイテムを保存
		}
	}
}

?>
